/***************************************************************************
                          funitem.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Aug 29 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FUNITEM_H
#define FUNITEM_H

#include <kplitem.h>

class KSimpleConfig;
class KplCheckListItem;

/**
  * Function item class. Class for function items showing ordinary functions
  * by lines or markers.
  * @author Werner Stille
  */

class FunItem : public ScaledItem {

public:
  /** Constructor */
	FunItem();
  /**
    * Constructor. Initializes corresponding to autoplot settings.
    * @param aut pointer to structure containing autoplot settings.
    */
  FunItem(Kpl::AutoStruct* aut);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param aut pointer to structure containing autoplot settings.
    * @param url URL of plot file.
    */
  FunItem(KSimpleConfig* plo, Kpl::AutoStruct* aut, const KURL& url);
  /** Destructor */
  virtual ~FunItem();
  virtual Kpl::ItemTypes iType() const;
  virtual void draw(KplGraph* g);
  virtual void writePlo(FILE* f, const KURL& url, bool abs) const;
  virtual void setText(KplCheckListItem* it, bool* arrays, bool* funcs) const;
  virtual int editItem(QWidget* parent, KplDoc* m, int i);
  virtual KplItem* copy() const;
  virtual void expoItem(int* iext, int* ieyt, double* fxt, double* fyt) const;
  virtual void minMax(double* xmi, double* xma, double* ymi,
                      double* yma) const;

protected:
  /**
   * Initializes several elements.
   */
  void init();
  /**
   * Closes shared library.
   */
  void freeX();
  /**
   * Opens shared library for copy of the item.
   * @param it pointer to copied item.
   */
  void copyX(FunItem* it) const;
  double tmin, tmax, dt, tmino, tmaxo, dto;
  double (*fkty)(double, const double*), (*fktyo)(double, const double*);
  double py[KPL_NPMAX], pyo[KPL_NPMAX];
  QArray<double> tv, yv;
  char namey[100], pathy[260];
  void* hmody;

  friend class FitDlg;
  friend class LMFit;
  friend class FuncDlg;
  friend class KplDoc;
};

/**
  * Parametric function item class.
  * @author Werner Stille
  */

class ParFunItem : public FunItem {

public:
  /** Constructor */
  ParFunItem();
  /**
    * Constructor. Initializes corresponding to autoplot settings.
    * @param aut pointer to structure containing autoplot settings.
    */
  ParFunItem(Kpl::AutoStruct* aut);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param aut pointer to structure containing autoplot settings.
    * @param url URL of plot file.
    */
  ParFunItem(KSimpleConfig* plo, Kpl::AutoStruct* aut, const KURL& url);
  /** Destructor */
  virtual ~ParFunItem();
  virtual Kpl::ItemTypes iType() const;
  virtual void draw(KplGraph* g);
  virtual void writePlo(FILE* f, const KURL& url, bool abs) const;
  virtual void setText(KplCheckListItem* it, bool* arrays, bool* funcs) const;
  virtual int editItem(QWidget* parent, KplDoc* m, int i);
  virtual KplItem* copy() const;
  virtual void expoItem(int* iext, int* ieyt, double* fxt, double* fyt) const;
  virtual void minMax(double* xmi, double* xma, double* ymi,
                      double* yma) const;

protected:
  /**
   * Initializes several elements.
   */
  void init();
  /**
   * Closes shared libraries.
   */
  void freeX();
  /**
   * Opens shared libraries for copy of the item.
   * @param it pointer to copied item.
   */
  void copyX(ParFunItem* it) const;
  double (*fktx)(double, const double*), (*fktxo)(double, const double*);
  double px[KPL_NPMAX], pxo[KPL_NPMAX];
  QArray<double> xv;
  char namex[100], pathx[260];
  void* hmodx;
  friend class ParFuncDlg;
};

#endif
